--------------------------------------------------------------------------------
-- Copyright  2019								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- For more information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...

local L
local ButtonXos = 20
local Container, LocalSettings
local ChannelIdList = {} 
local ChannelOrder = { Current=1, Original=1, }
local ChannelUpdates = { NewSettings={}, SortOrder={}, }

local FrameWidth = 290
local HeightExtra = 20
local MaxChannels = 7

local Delete, Save = 1, 2

local function ConfigureChat(parent)
	if Container then
		Container:SetShown(not Container:IsShown())
		return Container
	end

	L = NS.TEXTS
	local LSM = LibStub("LibSharedMedia-3.0")
	for k, v in pairs(LSM:List("font")) do
		NS.SharedMediaFonts[k] = { text=v }
	end

	local CheckButtons = {}
	local Controls = {}
	local function CopySettings(character, charsettings)
		local Settings = NS:GetSettings("Chat")
		local CharacterChat = charsettings.CHAT
		-- Not doing anything with "playback" settings (not used yet)
		for k,v in pairs(Settings) do
			if k ~= "channelbuttons" and type(v) ~= "table" then
				Settings[k] = nil
			end
		end
		for k,v in pairs(CharacterChat) do
			if k ~= "channelbuttons" and type(v) ~= "table" then
				Settings[k] = v
			end
		end
		for i=#Settings.channelbuttons, 1, -1 do
			if not CharacterChat.channelbuttons[i] then
				tremove(Settings.channelbuttons, i)
			end
		end
		for i=1, #CharacterChat.channelbuttons do
			table.wipe(Settings.channelbuttons[i])
			NS.Lib:CopyTable(CharacterChat.channelbuttons, Settings.channelbuttons)
			NS:SetChatWindowName(Settings.channelbuttons[i].id, Settings.channelbuttons[i].name)
		end
		NS:CopySettings(character, charsettings, "Chat", Controls, CheckButtons)
		NS:GetOption("ConfigureChatFrames")()
		Container:GetScript("OnShow")(Container)
	end

	local FontSizes = {
		{ text = L.Normal, value = 1 },
		{ text = L.EagleEye, value = 2 },
		{ text = L.Large, value = 3 },
		{ text = L.Magoo, value = 4 },
	}

	local WisperModes = {
		{ text = CONVERSATION_MODE_POPOUT, value = "popout", tooltip=OPTION_WHISPER_MODE_POPOUT, },
		{ text = CONVERSATION_MODE_INLINE, value = "inline", tooltip=OPTION_WHISPER_MODE_INLINE, },
		{ text = CONVERSATION_MODE_POPOUT_AND_INLINE, value = "popout_and_inline", tooltip=OPTION_WHISPER_MODE_POPOUT_AND_INLINE, },
	}

	local Settings = NS:GetSettings("Chat")

	local function InitialiseChatFrameSettings(self)
		for i=6, NUM_CHAT_WINDOWS do -- 5 is guild is permanent
			ChannelIdList[i] = true
		end
		for i=2, #LocalSettings do
			table.wipe(LocalSettings[i])
			if Settings.channelbuttons[i - 1] then
				NS.Lib:CopyTable(Settings.channelbuttons[i - 1], LocalSettings[i])
				LocalSettings[i].name = NS:GetChatWindowName(LocalSettings[i].id, LocalSettings[i].name)
				LocalSettings[i].channelbutton = Settings.channelbuttons[i - 1]
				LocalSettings[i].order = i - 1
				ChannelIdList[LocalSettings[i].id] = false
			else
				for j=6, NUM_CHAT_WINDOWS do
					if ChannelIdList[j] == true then
						LocalSettings[i].id = j
						ChannelIdList[j] = false
						break
					end
				end
				LocalSettings[i].name = NS:GetChatWindowName(LocalSettings[i].id)
				LocalSettings[i].ignore = true
				LocalSettings[i].empty = true
			end
		end
		self.FrameList:Init()
	end	

	Container = CreateFrame("Frame", "$parentChat", parent)

	Container.ChatFont = NS:CreateDropList("ChatFont", Container, Settings.chatfont, "font", NS.SharedMediaFonts, "text", "text")
	Container.ChatFont:SetLabel(L.Font, "TOP", 0, 3)
	Container.ChatFont:SetToolTip(L.FontTip)
	Container.ChatFont:SetSize(105, 23)
	Container.ChatFont:SetPoint("TOPRIGHT", Container, "TOP", -12, -55)
	Container.ChatFont.PostUpdate = function(self) 
		NS:GetOption("ConfigureChatSettings")(nil, nil, true)
	end
	Controls[Container.ChatFont] = "chatfont"

	Container.FontLabel = NS:CreateLabel(Container, "ChatFontSize", L.FontSize)
	Container.FontLabel:SetPoint("TOP", Container.ChatFont, "BOTTOM", 0, -10)

	Container.ChatSize = NS:CreateDropList("ChatFontSize", Container, Settings.chatfont, "size", FontSizes, "text", "value")
	Container.ChatSize:SetLabel(L.Chat, "LEFT", 0, 0)
	Container.ChatSize:SetToolTip(L.ChatFontSizeTip)
	Container.ChatSize:SetSize(80, 23)
--	Container.ChatSize:AddPadding(15)
	Container.ChatSize:SetPoint("TOPRIGHT", Container.ChatFont, "BOTTOMRIGHT", 0, -22)
	Container.ChatSize.PostUpdate = function(self) 
		NS:GetOption("ConfigureChatSettings")(nil, nil, true)
	end
	Controls[Container.ChatSize] = "chatfont"

	Container.EditSize = NS:CreateDropList("EditFontSize", Container, Settings.editfont, "size", FontSizes, "text", "value")
	Container.EditSize:SetLabel(L.Edit, "LEFT", 5, 0)
	Container.EditSize:SetToolTip(L.EditFontSizeTip)
	Container.EditSize:SetSize(80, 23)
--	Container.EditSize:AddPadding(15)
	Container.EditSize:SetPoint("TOP", Container.ChatSize, "BOTTOM", 0, -5)
	Container.EditSize.PostUpdate = function(self) 
		NS:GetOption("ConfigureChatSettings")(nil, nil, true)
	end
	Controls[Container.EditSize] = "editfont"

	Container.HideBar = NS.Lib:CreateLedCheckButton("HideBar", Container, Settings, "hide", "Black", "Green")
	Container.HideBar:SetSize(20, 20)
	NS:AddCheckLabel(Container.HideBar, L.ChatBarHide, "LEFT")
	Container.HideBar:SetToolTip(L.ChatBarHideTip)
	Container.HideBar:SetPoint("TOPLEFT", Container.EditSize, "BOTTOMLEFT", 0, -35)
	function Container.HideBar.PostUpdate(self)
		NS:GetOption("ChatSelectBarSetForHide")(self:GetChecked())
	end
	CheckButtons[Container.HideBar] = true -- true = use base settings, not branch

	Container.LockActive = NS.Lib:CreateLedCheckButton("LockActive", Container, Settings, "lockactive", "Green", "Black")
	Container.LockActive:SetSize(20, 20)
	NS:AddCheckLabel(Container.LockActive, L.Lock, "LEFT")
	Container.LockActive:SetToolTip(L.LockActiveChatTip)
	Container.LockActive:SetPoint("TOP", Container.HideBar, "BOTTOM", 0, -5)
	Container.LockActive.LockIcon = Container.LockActive:CreateTexture()
	Container.LockActive.LockIcon:SetSize(14, 14)
	Container.LockActive.LockIcon:SetPoint("RIGHT", Container.LockActive.Label, "LEFT", -3, 2)
	Container.LockActive.LockIcon:SetTexture(NS:GetOption("TexturePath").."Lock")
	function Container.LockActive.PostUpdate(self)
		NS:GetOption("SetChannelLock")()
	end
	CheckButtons[Container.LockActive] = true -- true = use base settings, not branch

	Container.AutoGuild = NS.Lib:CreateLedCheckButton("AutoGuild", Container, Settings, "autoguildchannel", "Green", "Black")
	Container.AutoGuild:SetSize(20, 20)
	NS:AddCheckLabel(Container.AutoGuild, L.Guild, "LEFT")
	Container.AutoGuild:SetToolTip(L.GuildChatTip)
	Container.AutoGuild:SetPoint("TOPRIGHT", Container.EditSize, "BOTTOMRIGHT", 0, -35)
	function Container.AutoGuild.PostUpdate(self)
		for i=1, #Settings.channelbuttons do
			if Settings.channelbuttons[i].id == 5 and not Settings.autoguildchannel then
				Settings.channelbuttons[i].ignore = nil
				break
			end
		end
--[[ add back if we ever have "Unmanaged" chat frames
		local Parent = self:GetParent()
		if ChannelUpdates.OrignalSettings and ChannelUpdates.OrignalSettings.id == 5 and Parent.Frame.Managed:IsShown() then
			Parent.Frame.Managed:Init()
			Parent.Frame.Managed:SetEnabled(not self:GetChecked())
		end
]]--
		NS:GetOption("AutoGuildUpdate")()
		InitialiseChatFrameSettings(Container)
		NS:GetOption("ConfigureChatSettings")()
	end
	CheckButtons[Container.AutoGuild] = true -- true = use base settings, not branch

	Container.PrintMoney = NS.Lib:CreateLedCheckButton("PrintMoney", Container, Settings, "printmoney", "Green", "Black")
	Container.PrintMoney:SetSize(20, 20)
	NS:AddCheckLabel(Container.PrintMoney, L.Money, "LEFT")
	Container.PrintMoney:SetToolTip(L.PrintMoneyTip)
	Container.PrintMoney:SetPoint("TOP", Container.AutoGuild, "BOTTOM", 0, -5)
	function Container.PrintMoney.PostUpdate(self)
		if self:GetChecked() then
			ChatFrame_RemoveMessageGroup(ChatFrame4, "MONEY")
		else
			ChatFrame_AddMessageGroup(ChatFrame4, "MONEY")
		end
		NS:GetOption("SetGoldOnly")()
	end
	CheckButtons[Container.PrintMoney] = true

--[[
	Container.MultiLine = NS.Lib:CreateLedCheckButton("MultiLine", Container, Settings, "multiline", "Green")
	Container.MultiLine:SetSize(20, 20)
	Container.MultiLine:SetPoint("TOPLEFT", 0, -30)
	Container.MultiLine:SetLabel(L.MultiLine, "RIGHT")
	Container.MultiLine:SetToolTip(L.MultiLineTip)
	Container.MultiLine.PostClick = NS:GetOption("SetEditBoxOptions")

	Container.PlayBack = NS:CreateLabel(Container, "PlayBack", L.Playback)
	Container.PlayBack:SetPoint("TOP", 0, -40)

	Container.PlayBackHistoryEnable = NS.Lib:CreateLedCheckButton("HistoryEnable", Container, Settings.playback.history, "enabled", "Green")
	Container.PlayBackHistoryEnable:SetPoint("TOPLEFT", 0, -45)
	Container.PlayBackHistoryEnable:SetLabel(L.History, "RIGHT")
	Container.PlayBackHistoryEnable:SetToolTip(L.HistoryTip)
	Container.PlayBackHistoryEnable.PostClick = NS:GetOption("PlaybackInitialise")

	Container.PlayBackHistoryWipe = NS.Lib:CreateLedCheckButton("HistoryWipe", Container, Settings.playback.history, "wipe", "Green")
	Container.PlayBackHistoryWipe:SetPoint("TOPRIGHT", 0, -45)
	Container.PlayBackHistoryWipe:SetLabel(L.HistoryWipe, "LEFT")
	Container.PlayBackHistoryWipe:SetToolTip(L.HistoryWipeTip)

	Container.PlayBackAFKEnable = NS.Lib:CreateLedCheckButton("AFKEnable", Container, Settings.playback.afk, "enabled", "Green")
	Container.PlayBackAFKEnable:SetPoint("TOPLEFT", Container.PlayBackHistoryEnable, "BOTTOMLEFT", 0, -5)
	Container.PlayBackAFKEnable:SetLabel(L.AFK, "RIGHT")
	Container.PlayBackAFKEnable:SetToolTip(L.AFKTip)
	Container.PlayBackAFKEnable.PostClick = NS:GetOption("PlaybackInitialise")
]]--

	if NS:IsValidVersion(WOW_PROJECT_MAINLINE) then
		Settings.whispermode = GetCVar("whisperMode")
		Container.WhisperMode = NS:CreateDropList("WhisperMode", Container, Settings, "whispermode", WisperModes, "text", "value")
		Container.WhisperMode:SetLabel(L.Whispers, "LEFT", 5, 0)
		Container.WhisperMode:SetToolTip(WisperModes[Container.WhisperMode.Key].tooltip)
		Container.WhisperMode:SetSize(80, 23)
--		Container.WhisperMode:AddPadding(15)
		Container.WhisperMode:SetPoint("TOPLEFT", Container.EditSize, "BOTTOMLEFT", 0, -5)
		Container.WhisperMode.PostUpdate = function(self) 
			SetCVar("whisperMode", self.Data[self.Index])
			Container.WhisperMode:SetToolTip(self.Table[self.Key].tooltip)
		end
		Container.WhisperMode:HookScript("OnShow", function(self)
			local Settings = NS:GetSettings("Chat")
			Settings.whispermode = GetCVar("whisperMode")
			self:Init()
		end)
		Controls[Container.WhisperMode] = "whispermode"
--	else
--		HeightExtra = 5
	end

	local ButtonSettings = { id="DelPage2", nolocale=true, text=L.GetText(L.Settings, L.Game), icon="Chat" }
	Container.ChatSettings = NS:CreateMenuButton(2, Container, "Button", ButtonSettings, 20, true)
	Container.ChatSettings:SetSize(95, 20)
	Container.ChatSettings:SetPoint("TOPLEFT", Container, ButtonXos, -8)
	Container.ChatSettings:SetScript("OnClick", function(self)
		ChatConfigFrame:SetShown(not ChatConfigFrame:IsShown())
	end)
	Container.ChatFont:AddDropHider(Container.FontLabel)
	Container.ChatFont:AddDropHider(Container.ChatSize)
	Container.ChatFont:AddDropHider(Container.EditSize)
	Container.ChatFont:AddDropHider(Container.HideBar)
	Container.ChatFont:AddDropHider(Container.AutoGuild)
	Container.ChatFont:AddDropHider(Container.LockActive)
	Container.ChatFont:AddDropHider(Container.PrintMoney)

	Container.ChatSize:AddDropHider(Container.EditSize)
	Container.ChatSize:AddDropHider(Container.HideBar)
	Container.ChatSize:AddDropHider(Container.AutoGuild)
	Container.ChatSize:AddDropHider(Container.LockActive)
	Container.ChatSize:AddDropHider(Container.PrintMoney)

	Container.EditSize:AddDropHider(Container.HideBar)
	Container.EditSize:AddDropHider(Container.AutoGuild)
	Container.EditSize:AddDropHider(Container.LockActive)
	Container.EditSize:AddDropHider(Container.PrintMoney)

	if NS:IsValidVersion(WOW_PROJECT_MAINLINE) then
		Container.ChatFont:AddDropHider(Container.WhisperMode)
		Container.ChatSize:AddDropHider(Container.WhisperMode)
		Container.EditSize:AddDropHider(Container.WhisperMode)

		Container.WhisperMode:AddDropHider(Container.HideBar)
		Container.WhisperMode:AddDropHider(Container.AutoGuild)
		Container.WhisperMode:AddDropHider(Container.LockActive)
		Container.WhisperMode:AddDropHider(Container.PrintMoney)
	end

-------------------------------------- Chat Channel Management --------------------------------------
	LocalSettings = {}
	for i=1, MaxChannels do
		tinsert(LocalSettings, {})
	end
	LocalSettings[1].id = 1
	LocalSettings[1].name = NS:GetChatWindowName(1, L.General)
	
	local function ChatButtonCreate(self)
		NS:SetScrollHighlight(self)
		self.LinkStatus = self:CreateTexture(nil, "OVERLAY", nil, 1)
		self.LinkStatus:SetSize(self:GetHeight() + 2, self:GetHeight() + 2)
		self.LinkStatus:SetPoint("LEFT", 5, 0)
		self.New = CreateFrame("Button", "#parentNew", self:GetParent())
		self.New:SetSize(self:GetHeight() + 9, self:GetHeight() + 9)
		self.New:SetNormalTexture(NS.TexturePath.."NewButton")
		self.New:SetPushedTexture(NS.TexturePath.."NewButtonPushed")
		self.New:SetHighlightTexture(self:GetHighlightTexture())
		self.New:SetPoint("RIGHT", self, -4, 0)
		self.New:SetFrameStrata(self:GetFrameStrata())
		self.New:SetFrameLevel(self:GetFrameLevel() + 3)
		self.New.Parent = self
		self.New:SetScript("OnClick", function(self)
			self.Parent:GetScript("OnClick")(self.Parent)
		end)
		NS.Lib:SetToolTip(self.New, L.ChatChannelNewTip)
		self.Text = NS.Lib:CreateFont("Text", self)
		self.Text:SetPoint("TOPLEFT", 23, -2)
		self.Text:SetPoint("BOTTOMRIGHT", self.New, 0, -2)
	end

	local function ChatButtonUpdate(self, table, index, data, savedvalue)
		if table[index].empty then
			self.Text:SetTextColor(0.5, 0.5, 0.5)
			self:SetEnabled(false)
			self:SetAlpha(0.8)
			self.New:Show()
		else
			if table[index].id == 1 or table[index].id == 5 then
				self.Text:SetTextColor(GameFontNormal:GetTextColor())
			else
				self.Text:SetTextColor(1, 1, 1)
			end
			self:SetEnabled(true)
			self:SetAlpha(1)
			self.New:Hide()
		end
		if table[index].ignore then
			self.LinkStatus:SetTexture(NS.Lib:GetTexturePath().."LEDRed")
		else
			self.LinkStatus:SetTexture(NS.Lib:GetTexturePath().."LEDGreen")
		end
	end

	local function InitialiseChatFrameSettings(self)
		for i=6, NUM_CHAT_WINDOWS do -- 5 is guild is permanent
			ChannelIdList[i] = true
		end
		for i=2, #LocalSettings do
			table.wipe(LocalSettings[i])
			if Settings.channelbuttons[i - 1] then
				NS.Lib:CopyTable(Settings.channelbuttons[i - 1], LocalSettings[i])
				LocalSettings[i].name = NS:GetChatWindowName(LocalSettings[i].id, LocalSettings[i].name)
				LocalSettings[i].channelbutton = Settings.channelbuttons[i - 1]
				LocalSettings[i].order = i - 1
				ChannelIdList[LocalSettings[i].id] = false
			else
				for j=6, NUM_CHAT_WINDOWS do
					if ChannelIdList[j] == true then
						LocalSettings[i].id = j
						ChannelIdList[j] = false
						break
					end
				end
				LocalSettings[i].name = NS:GetChatWindowName(LocalSettings[i].id)
				LocalSettings[i].ignore = true
				LocalSettings[i].empty = true
			end
		end
		self.FrameList:Init()
	end	
		
	local function TakeAction(self)
		local Parent = self:GetParent()
		local ContainerFrame = Parent:GetParent()
		local ConfigureChatFrames
		if self.SaveAction == Save then
			if ChannelUpdates.NewSettings.empty then -- Is new channel
				ConfigureChatFrames = true
				ChannelUpdates.OrignalSettings.empty = nil
				tinsert(Settings.channelbuttons, { id=ChannelUpdates.NewSettings.id})
				ChannelUpdates.OrignalSettings.channelbutton = Settings.channelbuttons[#Settings.channelbuttons]
-- This needs work for "UnDocked" Chat Frames
--[[
				if ChannelUpdates.NewSettings.ignore then
	local id = ChannelUpdates.NewSettings.id

	local _, _, _, _, _, _, shown = FCF_GetChatWindowInfo(id)
	local chatFrame = _G["ChatFrame"..id]
	local chatTab = _G["ChatFrame"..id.."Tab"]


	-- initialize the frame
	SetChatWindowLocked(id, false)
	FCF_SetWindowName(chatFrame, ChannelUpdates.NewSettings.name)
	FCF_SetWindowColor(chatFrame, DEFAULT_CHATFRAME_COLOR.r, DEFAULT_CHATFRAME_COLOR.g, DEFAULT_CHATFRAME_COLOR.b)
	FCF_SetWindowAlpha(chatFrame, DEFAULT_CHATFRAME_ALPHA)
--	FCF_UnDockFrame(chatFrame)

	FCF_DockFrame(chatFrame)

	-- clear stale messages
	chatFrame:Clear()

	-- Listen to the standard messages
	ChatFrame_RemoveAllMessageGroups(chatFrame)
	ChatFrame_RemoveAllChannels(chatFrame)
	ChatFrame_ReceiveAllPrivateMessages(chatFrame)
	SetChatWindowUninteractable(id, false)
	--Clear the edit box history.
	chatFrame.editBox:ClearHistory()

	-- Show the frame and tab
	chatFrame:Show()
	chatTab:Show()
	SetChatWindowShown(id, true)

	-- Dock the frame by default
--	FCF_DockFrame(chatFrame, (#FCFDock_GetChatFrames(GENERAL_CHAT_DOCK)+1), true);
--	FCF_FadeInChatFrame(FCFDock_GetSelectedWindow(GENERAL_CHAT_DOCK));
	ChatEdit_SetLastActiveWindow(chatFrame.editBox)

				end
]]--

			end
			if not ChannelUpdates.OrignalSettings.ignore and ChannelUpdates.OrignalSettings.ignore ~= ChannelUpdates.NewSettings.ignore then -- changed to unmanaged
				for i=1, #Settings.channelbuttons do
					if Settings.channelbuttons[i].id == ChannelUpdates.OrignalSettings.id then
						tremove(Settings.channelbuttons, i)
						tinsert(Settings.channelbuttons, ChannelUpdates.OrignalSettings.channelbutton)
						break
					end
				end
			elseif ChannelOrder.Original ~= ChannelOrder.New then
				for i=1, #Settings.channelbuttons do
					if Settings.channelbuttons[i].id == ChannelUpdates.OrignalSettings.id then
						tremove(Settings.channelbuttons, i)
						tinsert(Settings.channelbuttons, ChannelOrder.New - 1, ChannelUpdates.OrignalSettings.channelbutton)
						break
					end
				end
			end
			local Ignore
			if ChannelUpdates.NewSettings.ignore then
				Ignore = true
			end
			ChannelUpdates.OrignalSettings.channelbutton.ignore = Ignore
			ChannelUpdates.OrignalSettings.ignore = Ignore
			ChannelUpdates.OrignalSettings.channelbutton.name = ChannelUpdates.NewSettings.name
--			ChannelUpdates.OrignalSettings.name = ChannelUpdates.NewSettings.name
			NS:SetChatWindowName(ChannelUpdates.OrignalSettings.channelbutton.id, ChannelUpdates.OrignalSettings.channelbutton.name)
		elseif self.SaveAction == Delete then
			for i=1, #Settings.channelbuttons do
				if Settings.channelbuttons[i].id == ChannelUpdates.OrignalSettings.id then
					tremove(Settings.channelbuttons, i)
					ConfigureChatFrames = true
					break
				end
			end
		end
		NS:GetOption("ConfigureChatSettings")(nil, nil, true)
		if ConfigureChatFrames then
			NS:GetOption("ConfigureChatFrames")()
		end
		ChannelUpdates.OrignalSettings = nil
		Parent:Hide()
		InitialiseChatFrameSettings(ContainerFrame) -- Can change ChannelUpdates.OrignalSettings setting
		Parent:GetParent().FrameList:Init()
		Parent:GetParent().FrameList:Show()
	end

	--- Channel List
	Container.ChannelLabel = NS:CreateLabel(Container, "ChatChannelLabel", L.Channels)
	Container.ChannelLabel:SetPoint("TOPRIGHT", Container, "TOPRIGHT", -53, -45)

--	Container.ChannelsNotCopied = NS:CreateLabel(Container, "ChannelNoCopy", "("..L.NotCopied.."|cffffff00*|r)")
--	Container.ChannelsNotCopied:SetPoint("BOTTOM", Container.ChannelLabel, "TOP", 0, 5)

	Container.ChannelDummy = 1
	Container.FrameList = NS.Lib:CreateScrollList("Frames", Container, Container, "ChannelDummy", LocalSettings, "name", "name", MaxChannels, ChatButtonUpdate, ChatButtonCreate)
	Container.FrameList:SetSize(130, MaxChannels * 25)
	Container.FrameList:SetPoint("TOP", Container.ChannelLabel, "BOTTOM", 0, -10)
	function Container.FrameList:PostUpdate()
		self.Frame:Initialise(self.Table[self.Key])
	end

	--- Channel Update Form
	Container.Frame = CreateFrame("Frame", nil, Container)
	NS.AddBackground(Container.Frame)
	Container.Frame.Background:SetVertexColor(0.2, 0.2, 0.2, 0.4)
	Container.Frame:Hide()
	Container.Frame:SetPoint("TOPLEFT", Container.FrameList, 0, 0)
	Container.Frame:SetPoint("BOTTOMRIGHT", Container.FrameList)

	function Container.Frame:Initialise(entry)
		self.FrameList:Hide()
		ChannelUpdates.OrignalSettings = entry
		table.wipe(ChannelUpdates.NewSettings)
		NS.Lib:CopyTable(ChannelUpdates.OrignalSettings, ChannelUpdates.NewSettings)
		if ChannelUpdates.NewSettings.name == L.Unused then
			ChannelUpdates.NewSettings.name = L.New
		end
		if ChannelUpdates.OrignalSettings.empty then
			ChannelUpdates.NewSettings.ignore = nil -- Enables Managed button
		end
		self.Name:Init()
		ChannelOrder.New = 1
		table.wipe(ChannelUpdates.SortOrder)
		local Order = 1
		table.wipe(ChannelUpdates.SortOrder)
		for i=1, #Settings.channelbuttons do
--			if not Settings.channelbuttons[i].ignore then -- Ignored should go to the end of the list maybe??
				Order = Order + 1
				tinsert(ChannelUpdates.SortOrder, { order=Order })
				if ChannelUpdates.OrignalSettings.channelbutton == Settings.channelbuttons[i] then
					ChannelOrder.New = Order
				end
--			end
		end
		if ChannelUpdates.OrignalSettings.empty then
			Order = Order + 1
			tinsert(ChannelUpdates.SortOrder, { order=Order })
			ChannelOrder.New = Order
		end
		self.DeleteAction:SetChecked(false)
		self.DeleteAction:GetScript("OnClick")(self.DeleteAction)
		ChannelOrder.Original = ChannelOrder.New
		self.Name:SetEnabled(true)
		if ChannelUpdates.OrignalSettings.id == 1 then
--[[ add back if we ever have "Unmanaged" chat frames
			self.Managed:SetEnabled(false)
]]--
			self.DisplayOrder:SetEnabled(false)
		else
--[[ add back if we ever have "Unmanaged" chat frames
			self.Managed:SetEnabled(true)
			self.Managed:Init()
]]--
			self.DisplayOrder:Init()
		end
		if ChannelUpdates.OrignalSettings.id == 1 or ChannelUpdates.OrignalSettings.id == 5 then -- Can't delete General or Guild
			self.DeleteAction:SetEnabled(false)
--[[ add back if we ever have "Unmanaged" chat frames
			if ChannelUpdates.OrignalSettings.id == 5 and Settings.autoguildchannel then
				self.Managed:SetEnabled(false)
			end
]]--
		else
			self.DeleteAction:SetEnabled(true)
		end
		self:Show()
	end
	Container.FrameList.Frame = Container.Frame
	Container.Frame.FrameList = Container.FrameList
	
	Container.Frame.Name = NS.Lib:CreateEditBox("Name", Container.Frame, ChannelUpdates.NewSettings, "name", 90, 20, true)
	Container.Frame.Name:SetBackground("Interface\\BUTTONS\\WHITE8X8")
	Container.Frame.Name:SetBackgroundColor(0.06, 0.06, 0.06, 0.6)
	Container.Frame.Name:SetLabel(L.Name, "LEFT", 10, 0)
	Container.Frame.Name:SetToolTip(L.ChatNameTip)
	Container.Frame.Name:SetPoint("TOPRIGHT", -2, -8)

--/run FCF_OpenNewWindow("Crud", true) -- Copy relevant bits of this function to create a new channel  of the correct (FauxMazzle) ID
	
	local CheckSize = 20

	Container.Frame.Managed = NS.Lib:CreateLedCheckButton("Managed", Container.Frame, ChannelUpdates.NewSettings, "ignore", "Red", "Green")
	Container.Frame.Managed.UpdateOnInit = true
	Container.Frame.Managed:SetSize(CheckSize, CheckSize)
	Container.Frame.Managed:SetLabel(L.Button, "LEFT", 10, 0)
	Container.Frame.Managed:SetToolTip(L.ChatManagedTip)
Container.Frame.Managed:SetEnabled(false)
Container.Frame.Managed:Hide()
Container.Frame.Managed:SetPoint("BOTTOM", Container.Frame.Name, "BOTTOM", 0, 0) -- Temp SetPoint while not used
--[[ add back if we ever have "Unmanaged" chat frames
	Container.Frame.Managed:SetPoint("TOP", Container.Frame.Name, "BOTTOM", 0, -8)
	Container.Frame.Managed.PostUpdate = function(self)
		self:GetParent().DisplayOrder:SetEnabled(not self:GetChecked())
	end
]]--

	Container.Frame.DisplayOrder = NS:CreateDropList("DisplayOrder", Container.Frame, ChannelOrder, "New", ChannelUpdates.SortOrder, "order", "order", 3)
	Container.Frame.DisplayOrder:SetLabel(L.Order, "LEFT", 10, 0)
	Container.Frame.DisplayOrder:SetToolTip(L.ChatButtonOrderTip)
	Container.Frame.DisplayOrder:SetSize(65, 23)
	Container.Frame.DisplayOrder.Menu.ScrollList:SetHeight(50)
	Container.Frame.DisplayOrder:SetPoint("TOP", Container.Frame.Managed, "BOTTOM", 0, -8)

	Container.Frame.DeleteAction = CreateFrame("CheckButton", "$parentDeleteAction", Container.Frame)
	Container.Frame.DeleteAction:SetSize(20, 20)
	NS.Lib:AddLabelFuncs(Container.Frame.DeleteAction)
	Container.Frame.DeleteAction:SetLabel(L.Delete, "LEFT", 10, 0)
	Container.Frame.DeleteAction:SetToolTip(L.DeleteChannel)
	Container.Frame.DeleteAction:SetNormalTexture(NS.TexturePath .. "Delete1")
	Container.Frame.DeleteAction:SetCheckedTexture(NS.TexturePath .. "Delete")
	Container.Frame.DeleteAction:SetPoint("BOTTOMRIGHT", Container.Frame, 0, 34)
	Container.Frame.DeleteAction:GetNormalTexture():SetAlpha(0.8)
	Container.Frame.DeleteAction:GetCheckedTexture():SetAlpha(0.4)
	Container.Frame.DeleteAction:SetScript("OnClick", function(self)
		local Parent = self:GetParent()
		Parent.Save:SetShown(not self:GetChecked())
		Parent.Delete:SetShown(self:GetChecked())
		self:GetNormalTexture():SetAlpha(self:GetChecked() and 0.2 or 0.8)
		Parent.Name:SetEnabled(not self:GetChecked())
--[[ add back if we ever have "Unmanaged" chat frames
		Parent.Managed:SetEnabled(not self:GetChecked())
]]--
		Parent.DisplayOrder:SetEnabled(not self:GetChecked())
	end)
	Container.Frame.DisplayOrder:AddDropHider(Container.Frame.DeleteAction)

	--- Channel Update Form Action Buttons
	ButtonSettings = { id="Save", text="Save", icon="Save" }
	Container.Frame.Save = NS:CreateMenuButton(1, Container.Frame, "Button", ButtonSettings, 22, true)
	Container.Frame.Save:SetSize(65, 18)
	Container.Frame.Save.SaveAction = Save
	Container.Frame.Save:SetPoint("BOTTOMLEFT", Container.Frame, 2, 2)
	Container.Frame.Save:SetScript("OnClick", function(self)
		self:GetParent().Name:GetScript("OnEnterPressed")(self:GetParent().Name) -- Save the name in case they didn't press enter
		TakeAction(self)
	end)
	
	ButtonSettings = { id="Cancel", text="Cancel", icon="Cancel" }
	Container.Frame.Cancel = NS:CreateMenuButton(1, Container.Frame, "Button", ButtonSettings, 22, true)
	Container.Frame.Cancel:SetSize(65, 18)
	Container.Frame.Cancel:SetPoint("BOTTOMRIGHT", -2, 2)
	Container.Frame.Cancel:SetScript("OnClick", TakeAction)
	
	ButtonSettings = { id="Delete", text="Delete", icon="Delete1" }
	Container.Frame.Delete = NS:CreateMenuButton(1, Container.Frame, "CheckButton", ButtonSettings, 22, true)
	Container.Frame.Delete:Hide()
	Container.Frame.Delete:SetSize(65, 18)
	Container.Frame.Delete.SaveAction = Delete
	Container.Frame.Delete:SetAllPoints(Container.Frame.Save)
	Container.Frame.Delete:SetScript("OnClick", TakeAction)

	Container:HookScript("OnHide", function(self)
		local Parent = NS:GetFrame("Chat", "Parent")
		Parent:SetClicked()
	end)
	Container:HookScript("OnShow", function(self)
		InitialiseChatFrameSettings(self)
		TakeAction(Container.Frame.Cancel)
		NS:PlaceCopyButton(CopySettings, 10, 0, "TOPRIGHT", Container, "TOPRIGHT", -ButtonXos, -8)
	end)

--	Container:SetSize(FrameWidth, parent.Menu.MinHeight + HeightExtra)
	Container:SetSize(FrameWidth, Container.FrameList:GetHeight() * 1.35)
	Container:GetScript("OnShow")(Container)
	return Container
end

NS:RegisterModule("Chat", "Chat", "Chat", ConfigureChat)
